' Copyright 2009 Ivan Krivyakov
'
'   Licensed under the Apache License, Version 2.0 (the "License")
'   you may not use this file except in compliance with the License.
'   You may obtain a copy of the License at
'
'       http://www.apache.org/licenses/LICENSE-2.0
'
'   Unless required by applicable law or agreed to in writing, software
'   distributed under the License is distributed on an "AS IS" BASIS,
'   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
'   See the License for the specific language governing permissions and
'   limitations under the License.
' 

Imports System.Windows.Media.Media3D

Namespace Ikriv.Surfaces
	Public NotInheritable Class Circle
		Inherits Surface

		Private Shared RadiusProperty As New PropertyHolder(Of Double, Circle)("Radius", 1.0, AddressOf OnGeometryChanged)
		Public Property Radius As Double
			Get
				Return RadiusProperty.Get(Me)
			End Get
			Set(ByVal value As Double)
				RadiusProperty.Set(Me, value)
			End Set
		End Property

		Private Shared PositionProperty As New PropertyHolder(Of Point3D, Circle)("Position", New Point3D(0, 0, 0), AddressOf OnGeometryChanged)
		Public Property Position As Point3D
			Get
				Return PositionProperty.Get(Me)
			End Get
			Set(ByVal value As Point3D)
				PositionProperty.Set(Me, value)
			End Set
		End Property

		Private _radius As Double
		Private _position As Point3D

		Private Function PointForAngle(ByVal angle As Double) As Point3D
			Return New Point3D(_position.X + _radius*Math.Cos(angle), _position.Y + _radius*Math.Sin(angle), _position.Z)
		End Function

		Protected Overrides Function CreateMesh() As Geometry3D
			_radius = Radius
			_position = Position

			Dim mesh As New MeshGeometry3D
			Dim prevPoint As Point3D = PointForAngle(0)
			Dim normal As New Vector3D(0,0,1)

			Const div As Integer = 180
			For i  = 1 To div
				Dim angle As Double = 2 * Math.PI / div * i
				Dim newPoint As Point3D = PointForAngle(angle)
				mesh.Positions.Add(prevPoint)
				mesh.Positions.Add(_position)
				mesh.Positions.Add(newPoint)
				mesh.Normals.Add(normal)
				mesh.Normals.Add(normal)
				mesh.Normals.Add(normal)
				prevPoint = newPoint
			Next i

			mesh.Freeze()
			Return mesh
		End Function
	End Class
End Namespace
