' Copyright 2009 Ivan Krivyakov
'
'   Licensed under the Apache License, Version 2.0 (the "License")
'   you may not use this file except in compliance with the License.
'   You may obtain a copy of the License at
'
'       http://www.apache.org/licenses/LICENSE-2.0
'
'   Unless required by applicable law or agreed to in writing, software
'   distributed under the License is distributed on an "AS IS" BASIS,
'   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
'   See the License for the specific language governing permissions and
'   limitations under the License.
' 

Imports System.Windows.Media.Media3D

Namespace Ikriv.Surfaces
	Public NotInheritable Class Sphere
		Inherits Surface

		Private Shared RadiusProperty As New PropertyHolder(Of Double, Sphere)("Radius", 1.0, AddressOf OnGeometryChanged)
		Public Property Radius As Double
			Get
				Return RadiusProperty.Get(Me)
			End Get
			Set(ByVal value As Double)
				RadiusProperty.Set(Me, value)
			End Set
		End Property

		Private Shared PositionProperty As New PropertyHolder(Of Point3D, Sphere)("Position", New Point3D(0,0,0), AddressOf OnGeometryChanged)
		Public Property Position As Point3D
			Get
				Return PositionProperty.Get(Me)
			End Get
			Set(ByVal value As Point3D)
				PositionProperty.Set(Me, value)
			End Set
		End Property

		Private _radius As Double
		Private _position As Point3D

		Private Function GetPosition(ByVal angle As Double, ByVal y As Double) As Point3D
			Dim r As Double = _radius * Math.Sqrt(1 - y * y)
			Dim x As Double = r * Math.Cos(angle)
			Dim z As Double = r * Math.Sin(angle)

			Return New Point3D(_position.X + x, _position.Y + _radius*y, _position.Z + z)
		End Function

		Private Function GetNormal(ByVal angle As Double, ByVal y As Double) As Vector3D
			Return CType(GetPosition(angle, y), Vector3D)
		End Function

		Private Function GetTextureCoordinate(ByVal angle As Double, ByVal y As Double) As Point
			Dim map As New Matrix
			map.Scale(1 / (2 * Math.PI), -0.5)

			Dim p As New Point(angle, y)
			p = p * map

			Return p
		End Function

		Protected Overrides Function CreateMesh() As Geometry3D
			_radius = Radius
			_position = Position

			Const angleSteps As Integer = 32
			Const minAngle As Double = 0
			Const maxAngle As Double = 2 * Math.PI
			Const dAngle As Double = (maxAngle-minAngle) / angleSteps

			Const ySteps As Integer = 32
			Const minY As Double = -1.0
			Const maxY As Double = 1.0
			Const dy As Double = (maxY - minY) / ySteps

			Dim mesh As New MeshGeometry3D

			For yi  = 0 To ySteps
				Dim y As Double = minY + yi * dy

				For ai  = 0 To angleSteps
					Dim angle As Double = ai * dAngle

					mesh.Positions.Add(GetPosition(angle, y))
					mesh.Normals.Add(GetNormal(angle, y))
					mesh.TextureCoordinates.Add(GetTextureCoordinate(angle, y))
				Next ai
			Next yi

			For yi  = 0 To ySteps - 1
				For ai  = 0 To angleSteps - 1
					Dim a1 As Integer = ai
					Dim a2 As Integer = (ai + 1)
					Dim y1 As Integer = yi * (angleSteps + 1)
					Dim y2 As Integer = (yi + 1) * (angleSteps + 1)

					mesh.TriangleIndices.Add(y1 + a1)
					mesh.TriangleIndices.Add(y2 + a1)
					mesh.TriangleIndices.Add(y1 + a2)

					mesh.TriangleIndices.Add(y1 + a2)
					mesh.TriangleIndices.Add(y2 + a1)
					mesh.TriangleIndices.Add(y2 + a2)
				Next ai
			Next yi

			mesh.Freeze()
			Return mesh
		End Function
	End Class
End Namespace
