' Copyright 2009 Ivan Krivyakov
'
'   Licensed under the Apache License, Version 2.0 (the "License");
'   you may not use this file except in compliance with the License.
'   You may obtain a copy of the License at
'
'       http://www.apache.org/licenses/LICENSE-2.0
'
'   Unless required by applicable law or agreed to in writing, software
'   distributed under the License is distributed on an "AS IS" BASIS,
'   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
'   See the License for the specific language governing permissions and
'   limitations under the License.
' 

Imports System.Windows.Media.Media3D

Namespace Ikriv.Surfaces
	Public MustInherit Class Surface
		Inherits ModelVisual3D

		Public Sub New()
			Content = _content
			_content.Geometry = CreateMesh()
		End Sub

		Public Shared MaterialProperty As New PropertyHolder(Of Material, Surface)("Material", Nothing, AddressOf OnMaterialChanged)

		Public Shared BackMaterialProperty As New PropertyHolder(Of Material, Surface)("BackMaterial", Nothing, AddressOf OnBackMaterialChanged)

		Public Shared VisibleProperty As New PropertyHolder(Of Boolean, Surface)("Visible", True, AddressOf OnVisibleChanged)

		Public Property Material As Material
			Get
				Return MaterialProperty.Get(Me)
			End Get
			Set(ByVal value As Material)
				MaterialProperty.Set(Me, value)
			End Set
		End Property

		Public Property BackMaterial As Material
			Get
				Return BackMaterialProperty.Get(Me)
			End Get
			Set(ByVal value As Material)
				BackMaterialProperty.Set(Me, value)
			End Set
		End Property

		Public Property Visible As Boolean
			Get
				Return VisibleProperty.Get(Me)
			End Get
			Set(ByVal value As Boolean)
				VisibleProperty.Set(Me, value)
			End Set
		End Property

		Private Shared Sub OnMaterialChanged(ByVal sender As Object, ByVal e As DependencyPropertyChangedEventArgs)
			CType(sender, Surface).OnMaterialChanged
		End Sub

		Private Shared Sub OnBackMaterialChanged(ByVal sender As Object, ByVal e As DependencyPropertyChangedEventArgs)
			CType(sender, Surface).OnBackMaterialChanged
		End Sub

		Private Shared Sub OnVisibleChanged(ByVal sender As Object, ByVal e As DependencyPropertyChangedEventArgs)
			CType(sender, Surface).OnVisibleChanged
		End Sub

		Protected Shared Sub OnGeometryChanged(ByVal sender As Object, ByVal e As DependencyPropertyChangedEventArgs)
			CType(sender, Surface).OnGeometryChanged
		End Sub

		Private Sub OnMaterialChanged()
			SetContentMaterial()
		End Sub

		Private Sub OnBackMaterialChanged()
			SetContentBackMaterial()
		End Sub

		Private Sub OnVisibleChanged()
			SetContentMaterial()
			SetContentBackMaterial()
		End Sub

		Private Sub SetContentMaterial()
			_content.Material = If(Visible, Material, Nothing)
		End Sub

		Private Sub SetContentBackMaterial()
			_content.BackMaterial = If(Visible, BackMaterial, Nothing)
		End Sub

		Private Sub OnGeometryChanged()
			_content.Geometry = CreateMesh()
		End Sub

		Protected MustOverride Function CreateMesh() As Geometry3D

		Private ReadOnly _content As New GeometryModel3D
	End Class
End Namespace
