' Copyright (c) 2013 Ivan Krivyakov
' Licensed under the Apache License, Version 2.0 (the "License")
' you may not use this file except in compliance with the License.
' You may obtain a copy of the License at
' http://www.apache.org/licenses/LICENSE-2.0
'

Imports IKriv.WpfHost.Interfaces

Namespace UseLogService
	''' <summary>
	''' Interaction logic for MainUserControl.xaml
	''' </summary>
	Partial Public Class MainUserControl
		Inherits UserControl
		Implements IUnsavedData

		Private _textLogged As Boolean = True

		Public Sub New()
			InitializeComponent()
			Level.ItemsSource = System.Enum.GetValues(GetType(LogLevel))
			Level.SelectedItem = LogLevel.Debug
			AddHandler Message.TextChanged, AddressOf Message_TextChanged
		End Sub

		Private Sub Message_TextChanged(ByVal sender As Object, ByVal e As TextChangedEventArgs)
			_textLogged = False
			Logged.Visibility = Visibility.Hidden
		End Sub

		Public Property Log As ILog

        Public Function GetNamesOfUnsavedItems() As String() Implements IUnsavedData.GetNamesOfUnsavedItems
            If Not _textLogged Then
                Return {"Log message text"}
            End If
            Return Nothing
        End Function

		Private Sub LogIt_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			LogIt()
		End Sub

		Private Sub Message_KeyDown(ByVal sender As Object, ByVal e As KeyEventArgs)
			If e.Key = Key.Enter Then
				LogIt()
			End If
		End Sub

		Private Sub LogIt()
            Dim msg = "From UseLogService: " & Message.Text
			Select Case CType(Level.SelectedItem, LogLevel)
				Case LogLevel.Debug
                    Log.Debug(msg.ToString)
				Case LogLevel.Info
                    Log.Info(msg.ToString)
				Case LogLevel.Warning
                    Log.Warn(msg.ToString)
				Case LogLevel.Error
                    Log.Error(msg.ToString)
			End Select
			_textLogged = True
			Logged.Visibility = Visibility.Visible
		End Sub

		Private Sub UserControl_GotFocus(ByVal sender As Object, ByVal e As RoutedEventArgs)
			Message.Focus()
		End Sub
	End Class
End Namespace
