Imports IKriv.WpfHost.Interfaces

Namespace IKriv.PluginHosting.Log
	Public Class LocalLog
		Inherits MarshalByRefObject
		Implements ILog, IDisposable

		Private Structure AppenderInfo
			Public Sub New(ByVal appender As ILogAppender, ByVal level As LogLevel)
				Me.Appender = appender
				Me.Level = level
			End Sub

			Public Appender As ILogAppender
			Public Level As LogLevel
		End Structure

		Private ReadOnly _appenders As New List(Of AppenderInfo)

		Public Function Console(ByVal level As LogLevel) As LocalLog
			Return AddAppender(New ConsoleAppender, level)
		End Function

		Public Function Trace(ByVal name As String, ByVal level As LogLevel) As LocalLog
			Return AddAppender(New TraceAppender(name), level)
		End Function

		Public Function File(ByVal name As String, ByVal level As LogLevel) As LocalLog
			Return AddAppender(New TempFileAppender(name), level)
		End Function

		Public Function AddAppender(ByVal appender As ILogAppender, ByVal level As LogLevel) As LocalLog
			If level <> LogLevel.None Then
				_appenders.Add(New AppenderInfo(appender, level))
			End If
			Return Me
		End Function

        Public Sub Debug(ByVal message As String) Implements ILog.Debug
            Me.Message(LogLevel.Debug, message)
        End Sub

        Public Sub Info(ByVal message As String) Implements ILog.Info
            Me.Message(LogLevel.Info, message)
        End Sub

        Public Sub Warn(ByVal message As String) Implements ILog.Warn
            Me.Message(LogLevel.Warning, message)
        End Sub

        Public Sub Warn(ByVal message As String, ByVal ex As Exception) Implements ILog.Warn
            Me.Message(LogLevel.Warning, message & ". " & ex.ToString)
        End Sub

        Public Sub [Error](ByVal message As String) Implements ILog.Error
            Me.Message(LogLevel.Error, message)
        End Sub

        Public Sub [Error](ByVal message As String, ByVal ex As Exception) Implements ILog.Error
            Me.Message(LogLevel.Error, message & ". " & ex.ToString)
        End Sub

        Public Sub Dispose() Implements IDisposable.Dispose
            For Each appender In _appenders
                appender.Appender.Dispose()
            Next appender
        End Sub

        Public Overrides Function InitializeLifetimeService() As Object
            Return Nothing ' live forever
        End Function

        Private Sub Message(ByVal level As LogLevel, ByVal message As String)
            For Each appender In _appenders
                If appender.Level <= level Then
                    appender.Appender.Write(level.ToString.ToUpper & " " & message)
                End If
            Next appender
        End Sub
    End Class
End Namespace
