Imports System.IO

Namespace IKriv.PluginHosting.Log
	Friend Class TempFileAppender
		Implements ILogAppender

		Private ReadOnly _path As String
		Private _output As StreamWriter
		Private _failed As Boolean

		Public Sub New(ByVal name As String)
			Dim tempFolder = Path.GetTempPath
			Dim logFolder = Path.Combine(tempFolder, "WpfHost")
			_path = Path.Combine(logFolder, name & ".log")
		End Sub

		Public Sub Write(ByVal message As String) Implements ILogAppender.Write
			Try
				SyncLock Me
					If _output Is Nothing AndAlso (Not _failed) Then
						Dim folder = Path.GetDirectoryName(_path)
						Directory.CreateDirectory(folder)
						_output = New StreamWriter(_path, True)
					End If
				End SyncLock

				If _output IsNot Nothing Then
					_output.WriteLine(Date.Now.ToString("yyyy-MM-dd HH:mm:ss.fff") & " " & message)
				End If
			Catch ex As Exception
                Trace.WriteLine("Could not write to log file. " & ex.ToString)
				_failed = True
			End Try
		End Sub

		Public Sub Dispose() Implements IDisposable.Dispose
			Try
				If _output IsNot Nothing Then
					_output.Close()
				End If
			Catch ex As Exception
                Trace.WriteLine("Error closing log. " & ex.ToString)
			Finally
				_output = Nothing
			End Try
		End Sub
	End Class
End Namespace
