' Copyright (c) 2013 Ivan Krivyakov
' Licensed under the Apache License, Version 2.0 (the "License")
' you may not use this file except in compliance with the License.
' You may obtain a copy of the License at
' http://www.apache.org/licenses/LICENSE-2.0
'

Imports System.Reflection
Imports IKriv.WpfHost.Interfaces

Namespace IKriv.PluginHosting
	Friend NotInheritable Class PluginCreator
		Private Sub New()
		End Sub

		Public Shared Function CreatePlugin(ByVal assemblyName As String, ByVal typeName As String, ByVal host As IWpfHost) As Object
			Dim assembly = System.Reflection.Assembly.Load(assemblyName)
			Dim type = assembly.GetType(typeName)

			If type Is Nothing Then
				Throw New InvalidOperationException("Could not find type " & typeName & " in assembly " & assemblyName)
			End If

			SetupWpfApplication(assembly)
			Dim hostConstructor = type.GetConstructor( {GetType(IWpfHost)})
			If hostConstructor IsNot Nothing Then
				Return hostConstructor.Invoke(New Object(){host})
			End If

			Dim defaultConstructor = type.GetConstructor(New Type(){})
			If defaultConstructor Is Nothing Then
				Dim message = String.Format("Cannot create an instance of {0}. Either a public default constructor, or a public constructor taking IWpfHost must be defined", typeName)
				Throw New InvalidOperationException(message)
			End If

			Return defaultConstructor.Invoke(Nothing)
		End Function

		Private Shared Sub SetupWpfApplication(ByVal assembly As System.Reflection.Assembly)
			Dim application = New Application With {.ShutdownMode = ShutdownMode.OnExplicitShutdown}
			Application.ResourceAssembly = assembly
		End Sub
	End Class
End Namespace
