' Copyright (c) 2013 Ivan Krivyakov
' Licensed under the Apache License, Version 2.0 (the "License")
' you may not use this file except in compliance with the License.
' You may obtain a copy of the License at
' http://www.apache.org/licenses/LICENSE-2.0
'

Namespace IKriv.PluginHosting
	Public Class ProcessMonitor
		Private ReadOnly _onProcessExit As Action
		Private _fired As Boolean

		Public Sub New(ByVal onProcessExit As Action)
			If onProcessExit Is Nothing Then
				Throw New ArgumentNullException("onProcessExit")
			End If
			_onProcessExit = onProcessExit
		End Sub

		Public Sub Start(ByVal processId As Integer)
			Start(Process.GetProcessById(processId))
		End Sub

		Public Sub Start(ByVal process As Process)
			If process Is Nothing Then
				FireOnce()
				Exit Sub
			End If

			AddHandler process.Exited, Sub(sender, args) FireOnce()
			process.EnableRaisingEvents = True

			If process.HasExited Then
				FireOnce()
			End If
		End Sub

		Private Sub FireOnce()
			SyncLock Me
				If _fired Then
					Exit Sub
				End If
				_fired = True
			End SyncLock

			_onProcessExit()
		End Sub
	End Class
End Namespace
