' Copyright (c) 2013 Ivan Krivyakov
' Licensed under the Apache License, Version 2.0 (the "License")
' you may not use this file except in compliance with the License.
' You may obtain a copy of the License at
' http://www.apache.org/licenses/LICENSE-2.0
'

Imports System.Configuration
Imports System.IO
Imports IKriv.PluginHosting

Namespace IKriv.PluginProcess
	Friend Class Program
		<STAThread, LoaderOptimization(LoaderOptimization.MultiDomainHost)>
		Shared Sub Main(ByVal args() As String)
			Dim breakIntoDebugger As Boolean = Boolean.Parse(ConfigurationManager.AppSettings("BreakIntoDebugger"))
			If breakIntoDebugger Then
				Debugger.Break()
			End If

			Dim pauseOnError As Boolean = Boolean.Parse(ConfigurationManager.AppSettings("PauseOnError"))

			If args.Length <> 2 Then
				Console.Error.WriteLine("Usage: PluginProcess name assemblyPath")
				If pauseOnError Then
					Console.ReadLine()
				End If
				Exit Sub
			End If

			Try
				Dim name = args(0)
				Dim bits As Integer = IntPtr.Size * 8
				Console.WriteLine("Starting PluginProcess {0}, {1} bit", name, bits)

				Dim assemblyPath = args(1)
				Console.WriteLine("Plugin assembly: {0}", assemblyPath)

				CheckFileExists(assemblyPath)
				Dim configFile = GetConfigFile(assemblyPath)

				Dim appBase = Path.GetDirectoryName(assemblyPath)

				Dim appDomain = CreateAppDomain(appBase, configFile)
				Dim bootstrapper = CreateInstanceFrom(Of PluginLoaderBootstrapper)(appDomain)
				bootstrapper.Run(name)
			Catch ex As Exception
				Console.Error.WriteLine(ex)
				If pauseOnError Then
					Console.Error.WriteLine("Pausing on error, press any key to exit...")
					Console.ReadLine()
				End If
			End Try
		End Sub

		Private Shared Function CreateInstanceFrom(Of T)(ByVal appDomain As AppDomain) As T
			Return CType(appDomain.CreateInstanceFromAndUnwrap(GetType(T).Assembly.Location, GetType(T).FullName), T)
		End Function

		Private Shared Sub CheckFileExists(ByVal path As String)
			If Not File.Exists(path) Then
				Throw New InvalidOperationException("File '" & path & "' does not exist")
			End If
		End Sub

		Private Shared Function GetConfigFile(ByVal assemblyPath As String) As String
			Dim name = assemblyPath & ".config"
			Return If(File.Exists(name), name, Nothing)
		End Function

		Private Shared Function CreateAppDomain(ByVal appBase As String, ByVal config As String) As AppDomain
			Dim setup = New AppDomainSetup With {.ApplicationBase = appBase, .ConfigurationFile = If(String.IsNullOrEmpty(config), Nothing, config)}

			Return AppDomain.CreateDomain("PluginDomain", Nothing, setup)
		End Function
	End Class
End Namespace
