' Copyright (c) 2013 Ivan Krivyakov
' Licensed under the Apache License, Version 2.0 (the "License")
' you may not use this file except in compliance with the License.
' You may obtain a copy of the License at
' http://www.apache.org/licenses/LICENSE-2.0
'

Imports IKriv.PluginHosting.Log
Imports IKriv.WpfHost.Interfaces
Imports Microsoft.Practices.Unity

Namespace IKriv.WpfHost
	''' <summary>
	''' Interaction logic for App.xaml
	''' </summary>
	Partial Public Class App
		Inherits Application

		Private _container As IUnityContainer

		Protected Overrides Sub OnStartup(ByVal e As StartupEventArgs)
			_container = New UnityContainer
			ConfigureContainer()
			_container.Resolve(Of ILog)().Info("WpfHost starting")

			Dim mainWindow = _container.Resolve(Of MainWindow)()
			Dim viewModel = _container.Resolve(Of MainViewModel)()
			mainWindow.DataContext = viewModel
			mainWindow.Show()
		End Sub

		Protected Overrides Sub OnExit(ByVal e As ExitEventArgs)
			_container.Resolve(Of ILog)().Info("WpfHost shutting down")
			_container.Dispose()
			MyBase.OnExit(e)
		End Sub

        Private Sub ConfigureContainer()
            _container.RegisterType(Of ErrorHandlingService)(Singleton())
            _container.RegisterInstance(Of ILog)(New LocalLog().File("WpfHost", LogLevel.Debug), Singleton())
        End Sub

		Private Shared Function Singleton() As ContainerControlledLifetimeManager
			Return New ContainerControlledLifetimeManager
		End Function
	End Class
End Namespace
