' Copyright (c) 2013 Ivan Krivyakov
' Licensed under the Apache License, Version 2.0 (the "License")
' you may not use this file except in compliance with the License.
' You may obtain a copy of the License at
' http://www.apache.org/licenses/LICENSE-2.0
'

Imports IKriv.PluginHosting
Imports IKriv.WpfHost.Interfaces

Namespace IKriv.WpfHost
	Friend Class ErrorHandlingService
		Private ReadOnly _log As ILog

		Public Sub New(ByVal log As ILog)
			_log = log
		End Sub

		Public Sub ShowError(ByVal message As String, ByVal ex As Exception)
            _log.Error(message, ex)

			Dim text = If(ex Is Nothing, message, message & GetSeparator(message) & ExceptionUtil.GetUserMessage(ex))

			Dim mainWindow = GetMainWindow()
			If mainWindow Is Nothing Then
				MessageBox.Show(text, "Error", MessageBoxButton.OK, MessageBoxImage.Error)
			Else
				MessageBox.Show(mainWindow, text, "Error", MessageBoxButton.OK, MessageBoxImage.Error)
			End If
		End Sub

		Public Sub LogError(ByVal message As String, ByVal ex As Exception)
            _log.Error(message, ex)
		End Sub

		Public Function Confirm(ByVal message As String) As Boolean
			Return MessageBox.Show(message, "Please confirm", MessageBoxButton.OKCancel, MessageBoxImage.Question) = MessageBoxResult.OK
		End Function

		Private Shared Function GetMainWindow() As Window
			If Application.Current Is Nothing Then
				Return Nothing
			End If
			Return Application.Current.MainWindow
		End Function

		Private Shared Function GetSeparator(ByVal message As String) As String
			If message.EndsWith(".") Then
				Return " "
			End If
			If message.EndsWith(vbCrLf) Then
				Return ""
			End If
			Return ". "
		End Function
	End Class
End Namespace
