' Copyright (c) 2013 Ivan Krivyakov
' Licensed under the Apache License, Version 2.0 (the "License")
' you may not use this file except in compliance with the License.
' You may obtain a copy of the License at
' http://www.apache.org/licenses/LICENSE-2.0
'

Imports System.Windows.Controls.Primitives

Namespace IKriv.WpfHost
	''' <summary>
	''' Interaction logic for MainWindow.xaml
	''' </summary>
	Partial Public Class MainWindow
		Inherits Window

		Public Sub New()
			InitializeComponent()
		End Sub

		Private Sub OnLoadPluginClick(ByVal sender As Object, ByVal e As RoutedEventArgs)
			Dim viewModel = TryCast(DataContext, MainViewModel)
			If viewModel Is Nothing Then
				Exit Sub
			End If

			Dim menuItem = TryCast(sender, MenuItem)
			If menuItem Is Nothing Then
				Exit Sub
			End If

			Dim catalogEntry = TryCast(menuItem.CommandParameter, PluginCatalogEntry)
			If catalogEntry Is Nothing Then
				Exit Sub
			End If

			viewModel.LoadPluginCommand.Execute(catalogEntry)
		End Sub

		Private Sub Window_Closed(ByVal sender As Object, ByVal e As EventArgs)
			Dim viewModel = TryCast(DataContext, IDisposable)
			If viewModel Is Nothing Then
				Exit Sub
			End If
			viewModel.Dispose()
		End Sub

		Private Sub Window_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			' Move tab panel 25 pixels to the right
			' Cannot do it through style, because Margin is hard coded in the TabControl template
			Dim tabPanel = GetChildOfType(Of TabPanel)(PluginTabs)
			tabPanel.Margin = New Thickness(25,2,2,2)
		End Sub

		Private Shared Function GetChildOfType(Of T As DependencyObject)(ByVal depObj As DependencyObject) As T
			If depObj Is Nothing Then
				Return Nothing
			End If

			Dim count = VisualTreeHelper.GetChildrenCount(depObj)
			For i  = 0 To count - 1
				Dim child = VisualTreeHelper.GetChild(depObj, i)

				Dim result = If((TryCast(child, T)), GetChildOfType(Of T)(child))
				If result IsNot Nothing Then
					Return result
				End If
			Next i
			Return Nothing
		End Function

		Private Sub Window_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs)
			Dim viewModel = TryCast(DataContext, MainViewModel)
			If viewModel Is Nothing Then
				Exit Sub
			End If
			e.Cancel = Not viewModel.CanClose
		End Sub
	End Class
End Namespace
