' Copyright (c) 2013 Ivan Krivyakov
' Licensed under the Apache License, Version 2.0 (the "License")
' you may not use this file except in compliance with the License.
' You may obtain a copy of the License at
' http://www.apache.org/licenses/LICENSE-2.0
'

Namespace IKriv.WpfHost.Mvvm
    Public Class DelegateCommand(Of T)
        Implements ICommand

        Private ReadOnly _action As Action(Of T)
        Private ReadOnly _canExecute As Func(Of T, Boolean)

        Public Sub New(ByVal action As Action(Of T))
            Me.New(action, Nothing)
        End Sub

        Public Sub New(ByVal action As Action(Of T), ByVal canExecute As Func(Of T, Boolean))
            If action Is Nothing Then
                Throw New ArgumentNullException("action")
            End If
            _action = action
            _canExecute = If(canExecute, (Function(anything) True))
        End Sub

        Public Function CanExecute(ByVal parameter As Object) As Boolean Implements ICommand.CanExecute
            Return _canExecute(CType(parameter, T))
        End Function

        Public Sub Execute(ByVal parameter As Object) Implements ICommand.Execute
            _action(CType(parameter, T))
        End Sub

        Public Sub RaiseCanExecuteChanged()
            RaiseEvent CanExecuteChanged(Me, EventArgs.Empty)
        End Sub

        Public Event CanExecuteChanged As EventHandler Implements ICommand.CanExecuteChanged
    End Class

    Public Class DelegateCommand
        Inherits DelegateCommand(Of Object)

        Public Sub New(ByVal action As Action)
            Me.New(action, Nothing)
        End Sub

        Public Sub New(ByVal action As Action, ByVal canExecute As Func(Of Boolean))
            MyBase.New(EnsureNotNull(action), SafeConvert(canExecute))
        End Sub

        Private Shared Function EnsureNotNull(ByVal action As Action) As Action(Of Object)
            If action Is Nothing Then
                Throw New ArgumentNullException("action")
            End If
            Return Function(unusedArg) Sub() action()
        End Function

        Private Shared Function SafeConvert(ByVal canExecute As Func(Of Boolean)) As Func(Of Object, Boolean)
            If canExecute Is Nothing Then
                Return Nothing
            End If
            Return Function(unusedArg) canExecute()
        End Function
    End Class
End Namespace
